--[[
--CTF/Lua v2.0
--By Darren Watts (W'rkncacnter)
--]]

CAPTURES = 5
USE_FLAG_IN_BASE_RULE = true
team1_flag = nil --Polygon of team 1's flag
team2_flag = nil --Polygon of team 2's flag
team1_bases = {} --Polygons for team1
team2_bases = {} --Polygons for team2
teams = {}
team1_flag_in_base = true
team2_flag_in_base = true
player_has_flag = {}
count1 = 0
count2 = 0
wait = 1
points = {}
kills = {}
original_teams = {}
original_colors = {}
team1_color = 1
team2_color = 6
carrier_color = 4
finish_off = false
end_timer = 0
has_ended = false
holder_death_checked = {}
current_captures = {}
distance = {}
flag_position = {} -- first 3 elements are team 1's x, y, z. Next three are coords for team 2's flag.

--[[ ----------------  Icons ---------------------]]
bluesflag = [[
3
#5E2605
&0000FF
%111111
%##&%%%%%%%&&&&%
%##&&&%%%&&&&&&%
%##&&&&&&&&&&&&%
%##&&&&&&&&&&&&%
###&&&&&&&&&&&&%
##&&&&&&&&&&&&&%
##&&&&&&&&&&&&&%
##%%&&&&&&&%%%%%
##%%%%&&&%%%%%%%
##%%%%%%%%%%%%%%
##%%%%%%%%%%%%%%
##%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%
]];

redsflag = [[
3
#5e2605
&ff0000
%111111
%##&%%%%%%%&&&&%
%##&&&%%%&&&&&&%
%##&&&&&&&&&&&&%
%##&&&&&&&&&&&&%
###&&&&&&&&&&&&%
##&&&&&&&&&&&&&%
##&&&&&&&&&&&&&%
##%%&&&&&&&%%%%%
##%%%%&&&%%%%%%%
##%%%%%%%%%%%%%%
##%%%%%%%%%%%%%%
##%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%
]]

function init()
	initialize_vars()
	set_base_polys()
	check_teams()
	place_flags()
end

function initialize_vars()
	for i = 1, number_of_players(), 1 do
		player_has_flag[i] = false
		points[i] = 0
		holder_death_checked[i+1] = true
	end
	for i = 0, number_of_players(), 1 do
		kills[i] = {}
		for k = 0, number_of_players(), 1 do
			kills[i][k] = 0
		end
		distance[i+1] = 0
	end
	for i = 1, 6, 1 do
		flag_position[i] = 0
	end
	current_captures[1] = 0
	current_captures[2] = 0
end

function set_base_polys()
	for text,polygon,x,y in annotations() do
		if(string.lower(text) == "rf") then
			team1_flag = polygon
		end
		if(string.lower(text) == "bf") then
			team2_flag = polygon
		end
		if(string.lower(text) == "rb") then
			team1_bases[table.getn(team1_bases) + 1] = polygon
		end
		if(string.lower(text) == "bb") then
			team2_bases[table.getn(team2_bases) + 1] = polygon
		end
	end
	if(team1_flag == nil or team2_flag == nil or table.getn(team1_bases) == 0 
	or table.getn(team2_bases) == 0) then
		set_base_polys2()
	end
end

function set_base_polys2()
	if(team1_flag == nil) then
		for i = 0, number_of_polygons() - 1, 1 do
			if(get_polygon_type(i) == _polygon_is_visible_monster_trigger) then
				team1_flag = i
			end
		end
	end
	if(team2_flag == nil) then
		for i = 0, number_of_polygons() - 1, 1 do
			if(get_polygon_type(i) == _polygon_is_invisible_monster_trigger) then
				team2_flag = i
			end
		end
	end
	if(table.getn(team1_bases) == 0) then
		for i = 0, number_of_polygons() - 1, 1 do
			if(get_polygon_type(i) == _polygon_is_dual_monster_trigger) then
				team1_bases[table.getn(team1_bases) + 1] = i
			end
		end
	end
	if(table.getn(team2_bases) == 0) then
		for i = 0, number_of_polygons() - 1, 1 do
			if(get_polygon_type(i) == _polygon_must_be_explored) then
				team2_bases[table.getn(team2_bases) + 1] = i
			end
		end
	end
end

function check_teams()
	local red_team_exists = false
	local blue_team_exists = false
	for i = 0, number_of_players() - 1, 1 do
		if(get_player_team(i) == team1_color) then
			red_team_exists = true
		elseif(get_player_team(i) == team2_color) then
			blue_team_exists = true
		end
	end
	if(red_team_exists == false and blue_team_exists == false) then
		assign_teams(0)
	elseif(red_team_exists == true and blue_team_exists == false) then
		assign_teams(1)
	elseif(red_team_exists == false and blue_team_exists == true) then
		assign_teams(2)
	else
		assign_teams(3)
	end
end

function assign_teams(num)
	if(num == 0) then
		num = math.mod (better_random (), number_of_players())
		teams[1] = get_player_team(num)
		for i = 0, number_of_players() - 1, 1 do --Split into teams.
			if (get_player_team(i) ~= teams[1]) then
				teams[table.getn(teams) + 1] = get_player_team(i)
			end
		end
		for i = 0, number_of_players() - 1, 1 do
			if (get_player_team(i) == teams[1]) then
				set_player_team(i, team1_color)
				set_player_color(i, team1_color)
				original_teams[i+1] = team1_color
				original_colors[i+1] = get_player_color(i)
			else
				set_player_team(i, team2_color)
				set_player_color(i, team2_color)
				original_teams[i+1] = team2_color
				original_colors[i+1] = get_player_color(i)
			end
		end
	end
	if(num == 1 or num == 3) then
		for i = 0, number_of_players() - 1, 1 do
			if (get_player_team(i) == team1_color) then
				set_player_team(i, team1_color)
				set_player_color(i, team1_color)
				original_teams[i+1] = team1_color
				original_colors[i+1] = get_player_color(i)
			else
				set_player_team(i, team2_color)
				set_player_color(i, team2_color)
				original_teams[i+1] = team2_color
				original_colors[i+1] = get_player_color(i)
			end
		end
	end
	if(num == 2) then
		for i = 0, number_of_players() - 1, 1 do
			if(get_player_team(i) == team2_color) then
				set_player_team(i, team2_color)
				set_player_color(i, team2_color)
				original_teams[i+1] = team2_color
				original_colors[i+1] = get_player_color(i)
			else
				set_player_team(i, team1_color)
				set_player_color(i, team1_color)
				original_teams[i+1] = team1_color
				original_colors[i+1] = get_player_color(i)
			end
		end
	end
	teams[1] = team1_color
	teams[2] = team2_color
end

function players_to_bases()
	for i = 0, number_of_players() - 1, 1 do
		to_base(i, 0)
	end
end

function place_flags()
	new_item (_item_uplink_chip, team1_flag)
	new_item (_item_spht_door_key, team2_flag)
end

function idle()
	if(wait == 0) then
		players_to_bases()
	end
	use_oxygen()
	use_overlay()
	make_sure_ctf_map()
	check_player_land_location()
	captured_flag()
	update_count()
	make_sure_flag_exists()
	carrier_flash()
	compass_on()
	end_game()
	victory_sound()
end

function distance_from_player(player, ix, iy, iz)
	local px, py, pz = get_player_position(player);
	theDistance = math.sqrt((ix - px)^2 + (iy - py)^2 + (iz - pz)^2);
	return theDistance;
end

function use_oxygen()
	if(team1_flag_in_base) then
		flag_position[1], flag_position[2] = get_polygon_center(team1_flag)
		flag_position[1] = flag_position[1] / 1024
		flag_position[2] = flag_position[2] / 1024
		flag_position[3] = flag_position[3] / 1024
	end
	for p = 0, number_of_players() - 1, 1 do --If team 2 has flag.
		if(player_has_flag[p+1] and original_teams[p+1] == teams[2]) then
			flag_position[1], flag_position[2], flag_position[3] = get_player_position(p)
		end
	end
	if(team2_flag_in_base) then
		flag_position[4], flag_position[5] = get_polygon_center(team2_flag)
		flag_position[4] = flag_position[4] / 1024
		flag_position[5] = flag_position[5] / 1024
		flag_position[6] = flag_position[6] / 1024
	end
	for p = 0, number_of_players() - 1, 1 do --If team 1 has flag.
		if(player_has_flag[p+1] and original_teams[p+1] == teams[1]) then
			flag_position[4], flag_position[5], flag_position[6] = get_player_position(p)
		end
	end
	for i = 0, number_of_players() - 1, 1 do
		if(original_teams[i+1] == teams[1]) then
			distance[i+1] = distance_from_player(
				i, flag_position[4], flag_position[5], flag_position[6])
		else
			distance[i+1] = distance_from_player(
				i, flag_position[1], flag_position[2], flag_position[3])
		end
	end
	local temp
	for i = 0, number_of_players() - 1, 1 do
		temp = 40 - distance[i+1]
		if(temp < 0) then set_oxygen(i, 0)
		else
			set_oxygen(i, ( temp / 40 ) * 10800)
		end
	end
end

function use_overlay()
	local team1_has_flag = false
	local team2_has_flag = false
	for i = 0, number_of_players() - 1, 1 do
		if(original_teams[i+1] == teams[1] and player_has_flag[i+1]) then
			team1_has_flag = true
		elseif(original_teams[i+1] == teams[2] and player_has_flag[i+1]) then
			team2_has_flag = true
		end
	end
	set_overlay_color(0, 2)
	set_overlay_text(0, current_captures[1] .. " / " .. CAPTURES)
	set_overlay_color(1, 7)
	set_overlay_text(1, current_captures[2] .. " / " .. CAPTURES)
	if(team1_has_flag) then
		set_overlay_text(2, "Carrying Flag")
	elseif(team1_has_flag == false and team2_flag_in_base == false) then
		set_overlay_text(2, "Dropped Flag")
	elseif(team2_flag_in_base) then
		set_overlay_text(2, "Not Carrying Flag")
	end
	set_overlay_color(2, 2)
	set_overlay_color(3, 7)
	set_overlay_text(3, " ")
	if(team2_has_flag) then
		set_overlay_text(4, "Carrying Flag")
	elseif(team2_has_flag == false and team1_flag_in_base == false) then
		set_overlay_text(4, "Dropped Flag")
	elseif(team1_flag_in_base) then
		set_overlay_text(4, "Not Carrying Flag")
	end
	set_overlay_color(4, 7)
	set_overlay_icon(2, redsflag);
	set_overlay_icon(4, bluesflag);
end

function make_sure_ctf_map()
	if(team1_flag == nil or team2_flag == nil or table.getn(team1_bases) == 0 
	or table.getn(team2_bases) == 0) then
		screen_print("This is not a valid ctf map.")
	end
end

function check_player_land_location()
	for i = 0, number_of_players() - 1, 1 do
		if(player_has_flag[i+1] and original_teams[i+1] == teams[1]) then
			local a1, b1, c1 = get_player_external_velocity(i)
			local x1, y1, z1 = get_player_position(i)
			if (player_is_dead(i) and c1 == 0 and holder_death_checked[i+1]) then
				for p = 0, number_of_players() - 1, 1 do
					if(original_teams[p+1] == teams[2]) then
						set_lua_compass_beacon (p, x1, y1)
						set_lua_compass_state (p, _network_compass_use_beacon)
						use_lua_compass(p, true)
						flag_position[4] = x1
						flag_position[5] = y1
						flag_position[6] = z1
					end
				end
				holder_death_checked[i+1] = false
				player_has_flag[i+1] = false
				set_player_color(i, original_colors[i+1])
			end
		end
		if(player_has_flag[i+1] and original_teams[i+1] == teams[2]) then
			local a2, b2, c2 = get_player_external_velocity(i)
			local x2, y2, z2 = get_player_position(i)
			if (player_is_dead(i) and c2 == 0 and holder_death_checked[i+1]) then
				for p = 0, number_of_players() - 1, 1 do
					if(original_teams[p+1] == teams[1]) then
						set_lua_compass_beacon (p, x2, y2)
						set_lua_compass_state (p, _network_compass_use_beacon)
						use_lua_compass(p, true)
						flag_position[1] = x2
						flag_position[2] = y2
						flag_position[3] = z2
					end
				end
				holder_death_checked[i+1] = false
				player_has_flag[i+1] = false
				set_player_color(i, original_colors[i+1])
			end
		end
	end
end

function captured_flag()
	for i = 0, number_of_players() - 1, 1 do
		if(original_teams[i+1] == teams[1]) then
			if(USE_FLAG_IN_BASE_RULE) then
				if(player_has_flag[i+1] == true and get_player_polygon(i) == team1_flag
				and team1_flag_in_base and not player_is_dead(i)) then
					set_player_color(i, original_colors[i+1])
					player_has_flag[i+1] = false
					team2_flag_in_base = true
					points[i+1] = points[i+1] + 1
					current_captures[1] = current_captures[1] + 1
					remove_item (i, _item_spht_door_key)
					award_kills (i, i, -1)
					alert_captured(i, 1)
					check_if_end_game(1)
				end
			else
				if(player_has_flag[i+1] == true and get_player_polygon(i) == team1_flag
				and not player_is_dead(i)) then
					set_player_color(i, original_colors[i+1])
					player_has_flag[i+1] = false
					team2_flag_in_base = true
					points[i+1] = points[i+1] + 1
					current_captures[1] = current_captures[1] + 1
					remove_item (i, _item_spht_door_key)
					award_kills (i, i, -1)
					alert_captured(i, 1)
					check_if_end_game(1)
				end
			end
		else
			if(USE_FLAG_IN_BASE_RULE) then
				if(player_has_flag[i+1] == true and get_player_polygon(i) == team2_flag
				and team2_flag_in_base and not player_is_dead(i)) then
					set_player_color(i, original_colors[i+1])
					player_has_flag[i+1] = false
					team1_flag_in_base = true
					points[i+1] = points[i+1] + 1
					current_captures[2] = current_captures[2] + 1
					remove_item (i, _item_uplink_chip)
					award_kills (i, i, -1)
					alert_captured(i, 2)
					check_if_end_game(2)
				end
			else
				if(player_has_flag[i+1] == true and get_player_polygon(i) == team2_flag
				and not player_is_dead(i)) then
					set_player_color(i, original_colors[i+1])
					player_has_flag[i+1] = false
					team1_flag_in_base = true
					points[i+1] = points[i+1] + 1
					current_captures[2] = current_captures[2] + 1
					remove_item (i, _item_uplink_chip)
					award_kills (i, i, -1)
					alert_captured(i, 2)
					check_if_end_game(2)
				end
			end
		end
	end
end

function alert_captured(player, num)
	if(num == 1) then
		for i = 0, number_of_players() - 1, 1 do
			if(original_teams[i+1] == teams[1] and i == player) then
				screen_print(i, "You have captured the enemy flag!")
			elseif(original_teams[i+1] == teams[1] and i ~= player) then
				screen_print(i, "Your team has captured the enemy flag!")
			elseif(original_teams[i+1] == teams[2]) then
				screen_print(i, "The enemy has captured your flag!")
			end
		end
	else
		for i = 0, number_of_players() - 1, 1 do
			if(original_teams[i+1] == teams[2] and i == player) then
				screen_print(i, "You have captured the enemy flag!")
			elseif(original_teams[i+1] == teams[2] and i ~= player) then
				screen_print(i, "Your team has captured the enemy flag!")
			elseif(original_teams[i+1] == teams[1]) then
				screen_print(i, "The enemy has captured your flag!")
			end
		end
	end
end

function update_count()
	if(count1 > 0) then
		count1 = count1 - 1
	end
	if(count2 > 0) then
		count2 = count2 - 1
	end
	if(end_timer > 0) then
		end_timer = end_timer - 1
	end
	if(wait >= -1) then
		wait = wait - 1
	end
end

function make_sure_flag_exists()
	local flag1exists = false
	local flag2exists = false
	for i = 0, MAXIMUM_OBJECTS_PER_MAP-1, 1 do
		if (item_index_valid(i)) then
			if (get_item_type(i) == _item_uplink_chip) then
				flag1exists = true
			elseif (get_item_type(i) == _item_spht_door_key) then
				flag2exists = true
			end
		end
	end
	if(flag1exists == false and team1_flag_in_base == true and count1 <= 0) then
		new_item (_item_uplink_chip, team1_flag)
		count1 = 10
	end
	if(flag2exists == false and team2_flag_in_base == true and count2 <= 0) then
		new_item (_item_spht_door_key, team2_flag)
		count2 = 10
	end
end

function carrier_flash()
	for i = 0, number_of_players() - 1, 1 do
		if(player_has_flag[i+1] == true) then
			set_player_color(i, carrier_color);
		end
	end
end

function compass_on()
	local team1_has_flag = false
	local team2_has_flag = false
	local carrier_team1 = nil
	local carrier_team2 = nil
	for i = 0, number_of_players() - 1, 1 do
		if(player_has_flag[i+1] == true) then
			if(original_teams[i+1] == teams[1]) then
				carrier_team1 = i
				team1_has_flag = true
			else
				carrier_team2 = i
				team2_has_flag = true
			end
		end
	end
	for i = 0, number_of_players() - 1, 1 do
		if(original_teams[i+1] == teams[1]) then
			if(team2_has_flag == true) then
				x, y, z = get_player_position(carrier_team2)
				set_lua_compass_beacon (i, x, y)
				set_lua_compass_state (i, _network_compass_use_beacon)
				use_lua_compass(i, true)
			end
		end
		if(original_teams[i+1] == teams[2]) then
			if(team1_has_flag == true) then
				x, y, z = get_player_position(carrier_team1)
				set_lua_compass_beacon (i, x, y)
				set_lua_compass_state (i, _network_compass_use_beacon)
				use_lua_compass(i, true)
			end
		end
		if(player_has_flag[i+1] == false and original_teams[i+1] == teams[2]) then
			if(team1_has_flag == true) then
				x, y, z = get_player_position(carrier_team1)
				set_lua_compass_beacon (i, x, y)
				set_lua_compass_state (i, _network_compass_use_beacon)
				use_lua_compass(i, true)
			end
		end
		if(team1_flag_in_base == true) then
			if(original_teams[i+1] == teams[1]) then 
				use_lua_compass(i, false)
			end
		end
		if(team2_flag_in_base == true) then
			if(original_teams[i+1] == teams[2]) then 
				use_lua_compass(i, false)
			end
		end
	end
end

function got_item(type, player)
	if(original_teams[player+1] == teams[1]) then
		if(type == _item_spht_door_key) then
			player_has_flag[player+1] = true
			alert_teams(1)
			team2_flag_in_base = false
			holder_death_checked[player+1] = true
		end
		if(type == _item_uplink_chip and team1_flag_in_base == true) then
			deny_pickup(player)
		end
		if(type == _item_uplink_chip and team1_flag_in_base == false) then
			save_flag(player)
		end
	elseif(original_teams[player+1] == teams[2]) then
		if(type == _item_uplink_chip) then
			player_has_flag[player+1] = true
			alert_teams(2)
			team1_flag_in_base = false
			holder_death_checked[player+1] = true
		end
		if(type == _item_spht_door_key and team2_flag_in_base == true) then
			deny_pickup(player)
		end
		if(type == _item_spht_door_key and team2_flag_in_base == false) then
			save_flag(player)
		end
	end
end

function alert_teams(num)
	if(num == 1) then
		for i = 0, number_of_players() - 1, 1 do
			if(original_teams[i+1] == teams[1] and player_has_flag[i+1] == true) then
				screen_print(i, "You have the enemy flag!")
			elseif(original_teams[i+1] == teams[1] and player_has_flag[i+1] == false) then
				screen_print(i, "Your team has the enemy flag!")
			elseif(original_teams[i+1] == teams[2]) then
				screen_print(i, "Your flag has been taken!")
				play_sound(i, _snd_alarm, 1)
			end
		end
	else
		for i = 0, number_of_players() - 1, 1 do
			if(original_teams[i+1] == teams[2] and player_has_flag[i+1] == true) then
				screen_print(i, "You have the enemy flag!")
			elseif(original_teams[i+1] == teams[2] and player_has_flag[i+1] == false) then
				screen_print(i, "Your team has the enemy flag!")
			elseif(original_teams[i+1] == teams[1]) then
				screen_print(i, "Your flag has been taken!")
				play_sound(i, _snd_alarm, 1)
			end
		end
	end
end

function to_base(player, num_tried)
	if(num_tried < 11) then
		if original_teams[player+1] == teams[1] then
			local poly_has_player = false
			local poly = math.mod (better_random (), table.getn(team1_bases))
			if(num_tried >= 10) then
				teleport_player(player, team1_bases[poly+1])
			end
			for i = 0, number_of_players() - 1, 1 do
				if(get_player_polygon(i) == team1_bases[poly+1]) then
					poly_has_player = true
					break
				end
			end
			if(poly_has_player == false) then
				teleport_player(player, team1_bases[poly+1])
			else
				to_base(player, num_tried + 1)
			end
		else
			local poly_has_player = false
			local poly = math.mod (better_random (), table.getn(team2_bases))
			if(num_tried >= 10) then
				teleport_player(player, team2_bases[poly+1])
			end
			for i = 0, number_of_players() - 1, 1 do
				if(get_player_polygon(i) == team2_bases[poly+1]) then
					poly_has_player = true
					break
				end
			end
			if(poly_has_player == false) then
				teleport_player(player, team2_bases[poly+1])
			else
				to_base(player, num_tried + 1)
			end
		end
	end
end

function player_killed(player, aggressor_player, action, projectile)
	kills[aggressor_player+1][player+1] = kills[aggressor_player+1][player+1] + 1
	award_kills (aggressor_player, player, -1)
end


function player_revived(player)
	to_base(player, 0)
end

function check_if_end_game(num)
	local total_points = 0
	for i = 0, number_of_players() - 1, 1 do
		if(original_teams[i+1] == teams[num]) then
			total_points = total_points + points[i+1]
		end
	end
	if(total_points >= CAPTURES) then
		if(num == 1) then team = "RED" else team = "BLUE" end
		screen_print(" ")
		screen_print(team .. " team wins!")
		finish_off = true
		end_timer = 100
	end
end

function end_game()
	if(finish_off == true and end_timer <= 0 and has_ended == false) then
		for i = 0, number_of_players() - 1, 1 do
			 teleport_player_to_level (i, 256)
		end
		has_ended = true
	end
end

function deny_pickup(player)
	if(original_teams[player+1] == teams[1]) then
		remove_item(player, _item_uplink_chip)
	else
		remove_item(player, _item_spht_door_key)
	end
end

function save_flag(player)
	if(original_teams[player+1] == teams[1]) then
		new_item (_item_uplink_chip, team1_flag)
		team1_flag_in_base = true
		remove_item(player, _item_uplink_chip)
		save_flag_message(1)
	else
		new_item (_item_spht_door_key, team2_flag)
		team2_flag_in_base = true
		remove_item(player, _item_spht_door_key)
		save_flag_message(2)
	end
end

function save_flag_message(num)
	if(num == 1) then
		for i = 0, number_of_players() - 1, 1 do
			if(original_teams[i+1] == teams[1]) then
				screen_print(i, "Your flag has been returned!")
			elseif(original_teams[i+1] == teams[2]) then
				screen_print(i, "The enemy's flag was returned!")
			end
		end
	else
		for i = 0, number_of_players() - 1, 1 do
			if(original_teams[i+1] == teams[2]) then
				screen_print(i, "Your flag has been returned!")
			elseif(original_teams[i+1] == teams[1]) then
				screen_print(i, "The enemy's flag was returned!")
			end
		end
	end
end

function victory_sound()
	if(end_timer == 90) then
		play_sound(local_player_index(), _snd_adjust_volume, 1)
	elseif(end_timer == 80) then
		play_sound(local_player_index(), _snd_adjust_volume, 1)
	elseif(end_timer == 70) then
		play_sound(local_player_index(), _snd_adjust_volume, 1)
	end
end

function cleanup()
	for i = 0, number_of_players() - 1, 1 do
		for k = -1, number_of_players() - 1, 1 do
			temp = kills[i+1][k+1]
			for a = 0, temp - 1, 1 do
				award_kills(i,k,1)
			end
		end
	end
	for i = 0, number_of_players() - 1, 1 do
		for k = 0, number_of_players() - 1, 1 do
			set_kills(i, k, 0)
		end
		set_kills(i,i,points[i+1])
		set_player_color(i, original_colors[i+1])
		set_player_team(i, original_teams[i+1])
	end
end